#include <stdint.h>
#include <stddef.h>
#include <limits.h>
#if defined(__cplusplus)
extern "C" {
#endif
  
int strncmpci(const char * s1, const char * s2, size_t num);

#if defined(__cplusplus)
}
#endif
